<?php
/**
 * Handler: to manage Open Content requests
 */

namespace EwTools\Handler;

use EwTools\Support\Date;
use EwTools\Support\Str;
use \OCAPI;

/**
 * OpenContent
 *
 * @link    http://infomaker.se
 * @package EwTools\Handler
 * @since   EwTools\Handler\OpenContent 0.1
 */
class OpenContent {
    
    /**
     * The prefix for every cached search response
     *
     * @var string
     */
    protected static $cache_prefix = 'everyware_oc_search';
    
    /**
     * Default values for making text search requests against the OCAPI class
     *
     * @var array
     */
    protected static $default_search_data = [
        'q'                   => '',
        'start'               => 0,
        'limit'               => 15,
        'contenttypes'        => [],
        'properties'          => [],
        'sort.name'           => null,
        'sort.indexfield'     => null,
        'facet'               => null,
        'facet.limit'         => null,
        'facet.mincount'      => null,
        'facet.date.mincount' => null,
        'use_cache'           => true,
        'create_article'      => true
    ];
    
    /**
     * Default values for fetching suggestions from OCAPI class
     *
     * @var array
     */
    protected static $default_suggest_data = [
        'field'                => null,
        'q'                    => null,
        'incompleteWord'       => null,
        'incompleteWordInText' => null,
    ];
    
    /**
     * Instance of this class
     *
     * @var OpenContent
     */
    protected static $instance = null;
    
    /**
     * Instance of the OCAPI class provided by the Every plugin
     *
     * @var OCApi
     */
    private $oc_api;
    
    /**
     * Contains the query before finished
     *
     * @var string
     */
    public static $oc_query;
    
    /**
     * OpenContent constructor.
     * Uses dependency-injection
     *
     * @since 0.1
     *
     * @param OCAPI $oc_api
     */
    private function __construct( OCAPI $oc_api ) {
        $this->oc_api = $oc_api;
    }
    
    /**
     * Get instance of this class
     *
     * @since 0.1
     * @return OpenContent
     */
    public static function getInstance() {
        
        if( static::$instance === null ) {
            static::$instance = new static( new OCAPI );
        }
        
        return static::$instance;
    }
    
    /**
     * Return the requested key using the cache_prefix
     *
     * @param string $key
     *
     * @since 0.1
     * @return string
     */
    protected static function getCacheKey( $key = '' ) {
        return Str::snake( Str::prepend( static::$cache_prefix, $key, ' ' ) );
    }
    
    /**
     * Helper function for use of the OCAPI class
     *
     * @since 0.1
     * @return OCAPI
     */
    private static function ocApi() {
        return static::getInstance()->oc_api;
    }
    
    /**
     * Function for creating text-search queries against OC using OCAPI class
     *
     * @use      OCAPI
     *
     * @param array $options {
     * Optional. Array of options to send to Open Content.
     *
     * @type string $q
     * @type int $start
     * @type int $limit
     * @type array[string] $contenttypes
     * @type array[string] $properties
     * @type string $sort.name
     * @type string $sort.indexfield
     * @type bool $facet
     * @type int $facet.limit
     * @type int $facet.mincount
     * @type int $facet.date.mincount
     * @type bool $use_cache
     * @type bool $create_article
    ];}
     * @since    0.1
     * @return array
     */
    public static function search( array $options = [] ) {
        
        $options = static::getOptions($options);
        
        $use_cache      = $options[ 'use_cache' ];
        $create_article = $options[ 'create_article' ];
        
        unset( $options[ 'use_cache' ], $options[ 'create_article' ] );
        return static::ocApi()->search( $options, $use_cache, $create_article );
    }
    
    /**
     * Function for fetching article suggestions
     *
     * @uses  OCAPI
     *
     * @param array $args {
     * Optional. Array of arguments to retrieve suggestions.
     *
     * @type array[string] $field
     * @type string $q
     * @type array[string] $incompleteWord
     * @type string $incompleteWordInText
     * }
     * @since 0.1
     * @return null|string
     */
    public static function suggest( array $args = [] ) {
        return call_user_func_array( [
            static::ocApi(),
            'ajax_get_suggest'
        ], array_replace( static::$default_suggest_data, $args ) );
    }
    
    /**
     * Fetch single article from id
     *
     * @uses  OCAPI
     *
     * @param string $uuid
     * @param array  $properties List off properties to fetch from Open Content
     *
     * @since 0.1
     * @return \OcArticle|null
     */
    public static function getArticle( $uuid, $properties = [] ) {
        
        $article = static::ocApi()->get_single_article( $uuid, $properties );
        
        return isset( $article[ 'article' ] ) ? $article[ 'article' ] : null;
    }
    
    /**
     * Creates a Solr-block of properties with the option to change the seperatoer
     *
     * @param string $property_name
     * @param array  $properties
     * @param string $glue
     *
     * @since 0.1
     * @return string
     */
    public static function addPropertyBlock( $property_name = '', $properties, $glue = 'OR' ) {
        return OpenContentQueryBuilder::propertySearch( $property_name, $properties, $glue );
    }
    
    /**
     * Create a Solr-property string
     *
     * @uses  Str::contains()
     *
     * @param $property
     * @param $property_name
     *
     * @since 0.1
     * @return string
     */
    public static function createProperty( $property, $property_name ) {
        return OpenContentQueryBuilder::createProperty( $property, $property_name );
    }
    
    protected static function getOptions( array $options= [] ) {
        return array_replace( static::$default_search_data, $options );
    }
}